/* SPDX-License-Identifier: GPL-2.0 */
/*
 * L2TP-over-IP socket for L2TPv3.
 *
 * Author: James Chapman <jchapman@katalix.com>
 */
#ifndef _LINUX_L2TP_H_
#define _LINUX_L2TP_H_

#include <linux/in.h>
#include <linux/in6.h>
#include <uapi/linux/l2tp.h>

#endif
