/* SPDX-License-Identifier: GPL-2.0+ */

#ifndef __DT_BINDINGS_RESET_BCM6358_H
#define __DT_BINDINGS_RESET_BCM6358_H

#define BCM6358_RST_SPI		0
#define BCM6358_RST_ENET	2
#define BCM6358_RST_MPI		3
#define BCM6358_RST_EPHY	6
#define BCM6358_RST_SAR		7
#define BCM6358_RST_USBH	12
#define BCM6358_RST_PCM		13
#define BCM6358_RST_ADSL	14

#endif /* __DT_BINDINGS_RESET_BCM6358_H */
