/* SPDX-License-Identifier: GPL-2.0 WITH Linux-syscall-note */
/* industrial I/O buffer definitions needed both in and out of kernel
 */

#ifndef _UAPI_IIO_BUFFER_H_
#define _UAPI_IIO_BUFFER_H_

#define IIO_BUFFER_GET_FD_IOCTL			_IOWR('i', 0x91, int)

#endif /* _UAPI_IIO_BUFFER_H_ */
